@@ -1,47 +1,34 @@
-/*
-    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
-
-    This file is part of libzmq, the ZeroMQ core engine in C++.
-
-    libzmq is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License (LGPL) as published
-    by the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    As a special exception, the Contributors give you permission to link
-    this library with independent modules to produce an executable,
-    regardless of the license terms of these independent modules, and to
-    copy and distribute the resulting executable under terms of your choice,
-    provided that you also meet, for each linked independent module, the
-    terms and conditions of the license of that module. An independent
-    module is a module which is not derived from or based on this library.
-    If you modify this library, you must extend this exception to your
-    version of the library.
-
-    libzmq is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
-    License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+/* SPDX-License-Identifier: MPL-2.0 */
 
 #include "precompiled.hpp"
 #include "ip.hpp"
 #include "err.hpp"
 #include "macros.hpp"
 #include "config.hpp"
+#include "address.hpp"
 
 #if !defined ZMQ_HAVE_WINDOWS
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#include <stdlib.h>
+#include <unistd.h>
+
+#include <vector>
 #else
 #include "tcp.hpp"
+#ifdef ZMQ_HAVE_IPC
+#include "ipc_address.hpp"
+#endif
+
+#include <direct.h>
+
+#define rmdir _rmdir
+#define unlink _unlink
 #endif
 
 #if defined ZMQ_HAVE_OPENVMS || defined ZMQ_HAVE_VXWORKS
@@ -70,6 +57,14 @@
 #include <TargetConditionals.h>
 #endif
 
+#ifndef ZMQ_HAVE_WINDOWS
+// Acceptable temporary directory environment variables
+static const char *tmp_env_vars[] = {
+  "TMPDIR", "TEMPDIR", "TMP",
+  0 // Sentinel
+};
+#endif
+
 zmq::fd_t zmq::open_socket (int domain_, int type_, int protocol_)
 {
     int rc;
@@ -84,7 +79,7 @@ zmq::fd_t zmq::open_socket (int domain_, int type_, int protocol_)
     // if supported, create socket with WSA_FLAG_NO_HANDLE_INHERIT, such that
     // the race condition in making it non-inheritable later is avoided
     const fd_t s = WSASocket (domain_, type_, protocol_, NULL, 0,
-                              WSA_FLAG_NO_HANDLE_INHERIT);
+                              WSA_FLAG_OVERLAPPED | WSA_FLAG_NO_HANDLE_INHERIT);
 #else
     const fd_t s = socket (domain_, type_, protocol_);
 #endif
@@ -108,7 +103,7 @@ void zmq::unblock_socket (fd_t s_)
 {
 #if defined ZMQ_HAVE_WINDOWS
     u_long nonblock = 1;
-    int rc = ioctlsocket (s_, FIONBIO, &nonblock);
+    const int rc = ioctlsocket (s_, FIONBIO, &nonblock);
     wsa_assert (rc != SOCKET_ERROR);
 #elif defined ZMQ_HAVE_OPENVMS || defined ZMQ_HAVE_VXWORKS
     int nonblock = 1;
@@ -127,14 +122,15 @@ void zmq::enable_ipv4_mapping (fd_t s_)
 {
     LIBZMQ_UNUSED (s_);
 
-#if defined IPV6_V6ONLY && !defined ZMQ_HAVE_OPENBSD
+#if defined IPV6_V6ONLY && !defined ZMQ_HAVE_OPENBSD                           \
+  && !defined ZMQ_HAVE_DRAGONFLY
 #ifdef ZMQ_HAVE_WINDOWS
     DWORD flag = 0;
 #else
     int flag = 0;
 #endif
-    int rc = setsockopt (s_, IPPROTO_IPV6, IPV6_V6ONLY,
-                         reinterpret_cast<char *> (&flag), sizeof (flag));
+    const int rc = setsockopt (s_, IPPROTO_IPV6, IPV6_V6ONLY,
+                               reinterpret_cast<char *> (&flag), sizeof (flag));
 #ifdef ZMQ_HAVE_WINDOWS
     wsa_assert (rc != SOCKET_ERROR);
 #else
@@ -145,39 +141,29 @@ void zmq::enable_ipv4_mapping (fd_t s_)
 
 int zmq::get_peer_ip_address (fd_t sockfd_, std::string &ip_addr_)
 {
-    int rc;
     struct sockaddr_storage ss;
 
-#if defined ZMQ_HAVE_HPUX || defined ZMQ_HAVE_WINDOWS                          \
-  || defined ZMQ_HAVE_VXWORKS
-    int addrlen = static_cast<int> (sizeof ss);
-#else
-    socklen_t addrlen = sizeof ss;
-#endif
-    rc = getpeername (sockfd_, reinterpret_cast<struct sockaddr *> (&ss),
-                      &addrlen);
+    const zmq_socklen_t addrlen =
+      get_socket_address (sockfd_, socket_end_remote, &ss);
+
+    if (addrlen == 0) {
 #ifdef ZMQ_HAVE_WINDOWS
-    if (rc == SOCKET_ERROR) {
         const int last_error = WSAGetLastError ();
         wsa_assert (last_error != WSANOTINITIALISED && last_error != WSAEFAULT
                     && last_error != WSAEINPROGRESS
                     && last_error != WSAENOTSOCK);
-        return 0;
-    }
-#else
-    if (rc == -1) {
-#if !defined(TARGET_OS_IPHONE) || !TARGET_OS_IPHONE
+#elif !defined(TARGET_OS_IPHONE) || !TARGET_OS_IPHONE
         errno_assert (errno != EBADF && errno != EFAULT && errno != ENOTSOCK);
 #else
         errno_assert (errno != EFAULT && errno != ENOTSOCK);
 #endif
         return 0;
     }
-#endif
 
     char host[NI_MAXHOST];
-    rc = getnameinfo (reinterpret_cast<struct sockaddr *> (&ss), addrlen, host,
-                      sizeof host, NULL, 0, NI_NUMERICHOST);
+    const int rc =
+      getnameinfo (reinterpret_cast<struct sockaddr *> (&ss), addrlen, host,
+                   sizeof host, NULL, 0, NI_NUMERICHOST);
     if (rc != 0)
         return 0;
 
@@ -217,6 +203,19 @@ void zmq::set_ip_type_of_service (fd_t s_, int iptos_)
 #endif
 }
 
+void zmq::set_socket_priority (fd_t s_, int priority_)
+{
+#ifdef ZMQ_HAVE_SO_PRIORITY
+    int rc =
+      setsockopt (s_, SOL_SOCKET, SO_PRIORITY,
+                  reinterpret_cast<char *> (&priority_), sizeof (priority_));
+    errno_assert (rc == 0);
+#else
+    LIBZMQ_UNUSED (s_);
+    LIBZMQ_UNUSED (priority_);
+#endif
+}
+
 int zmq::set_nosigpipe (fd_t s_)
 {
 #ifdef SO_NOSIGPIPE
@@ -237,20 +236,23 @@ int zmq::set_nosigpipe (fd_t s_)
     return 0;
 }
 
-void zmq::bind_to_device (fd_t s_, std::string &bound_device_)
+int zmq::bind_to_device (fd_t s_, const std::string &bound_device_)
 {
 #ifdef ZMQ_HAVE_SO_BINDTODEVICE
     int rc = setsockopt (s_, SOL_SOCKET, SO_BINDTODEVICE,
                          bound_device_.c_str (), bound_device_.length ());
+    if (rc != 0) {
+        assert_success_or_recoverable (s_, rc);
+        return -1;
+    }
+    return 0;
 
-#ifdef ZMQ_HAVE_WINDOWS
-    wsa_assert (rc != SOCKET_ERROR);
-#else
-    errno_assert (rc == 0);
-#endif
 #else
     LIBZMQ_UNUSED (s_);
     LIBZMQ_UNUSED (bound_device_);
+
+    errno = ENOTSUP;
+    return -1;
 #endif
 }
 
@@ -281,12 +283,12 @@ bool zmq::initialize_network ()
 #endif
 
 #ifdef ZMQ_HAVE_WINDOWS
-    //  Intialise Windows sockets. Note that WSAStartup can be called multiple
+    //  Initialise Windows sockets. Note that WSAStartup can be called multiple
     //  times given that WSACleanup will be called for each WSAStartup.
 
-    WORD version_requested = MAKEWORD (2, 2);
+    const WORD version_requested = MAKEWORD (2, 2);
     WSADATA wsa_data;
-    int rc = WSAStartup (version_requested, &wsa_data);
+    const int rc = WSAStartup (version_requested, &wsa_data);
     zmq_assert (rc == 0);
     zmq_assert (LOBYTE (wsa_data.wVersion) == 2
                 && HIBYTE (wsa_data.wVersion) == 2);
@@ -299,7 +301,7 @@ void zmq::shutdown_network ()
 {
 #ifdef ZMQ_HAVE_WINDOWS
     //  On Windows, uninitialise socket layer.
-    int rc = WSACleanup ();
+    const int rc = WSACleanup ();
     wsa_assert (rc != SOCKET_ERROR);
 #endif
 
@@ -314,36 +316,16 @@ void zmq::shutdown_network ()
 static void tune_socket (const SOCKET socket_)
 {
     BOOL tcp_nodelay = 1;
-    int rc =
+    const int rc =
       setsockopt (socket_, IPPROTO_TCP, TCP_NODELAY,
                   reinterpret_cast<char *> (&tcp_nodelay), sizeof tcp_nodelay);
     wsa_assert (rc != SOCKET_ERROR);
 
     zmq::tcp_tune_loopback_fast_path (socket_);
 }
-#endif
 
-int zmq::make_fdpair (fd_t *r_, fd_t *w_)
+static int make_fdpair_tcpip (zmq::fd_t *r_, zmq::fd_t *w_)
 {
-#if defined ZMQ_HAVE_EVENTFD
-    int flags = 0;
-#if defined ZMQ_HAVE_EVENTFD_CLOEXEC
-    //  Setting this option result in sane behaviour when exec() functions
-    //  are used. Old sockets are closed and don't block TCP ports, avoid
-    //  leaks, etc.
-    flags |= EFD_CLOEXEC;
-#endif
-    fd_t fd = eventfd (0, flags);
-    if (fd == -1) {
-        errno_assert (errno == ENFILE || errno == EMFILE);
-        *w_ = *r_ = -1;
-        return -1;
-    } else {
-        *w_ = *r_ = fd;
-        return 0;
-    }
-
-#elif defined ZMQ_HAVE_WINDOWS
 #if !defined _WIN32_WCE && !defined ZMQ_HAVE_WINDOWS_UWP
     //  Windows CE does not manage security attributes
     SECURITY_DESCRIPTOR sd;
@@ -370,9 +352,9 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
     //  Create critical section only if using fixed signaler port
     //  Use problematic Event implementation for compatibility if using old port 5905.
     //  Otherwise use Mutex implementation.
-    int event_signaler_port = 5905;
+    const int event_signaler_port = 5905;
 
-    if (signaler_port == event_signaler_port) {
+    if (zmq::signaler_port == event_signaler_port) {
 #if !defined _WIN32_WCE && !defined ZMQ_HAVE_WINDOWS_UWP
         sync =
           CreateEventW (&sa, FALSE, TRUE, L"Global\\zmq-signaler-port-sync");
@@ -385,14 +367,14 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
                                L"Global\\zmq-signaler-port-sync");
 
         win_assert (sync != NULL);
-    } else if (signaler_port != 0) {
+    } else if (zmq::signaler_port != 0) {
         wchar_t mutex_name[MAX_PATH];
 #ifdef __MINGW32__
         _snwprintf (mutex_name, MAX_PATH, L"Global\\zmq-signaler-port-%d",
-                    signaler_port);
+                    zmq::signaler_port);
 #else
         swprintf (mutex_name, MAX_PATH, L"Global\\zmq-signaler-port-%d",
-                  signaler_port);
+                  zmq::signaler_port);
 #endif
 
 #if !defined _WIN32_WCE && !defined ZMQ_HAVE_WINDOWS_UWP
@@ -413,7 +395,7 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
 
     //  Create listening socket.
     SOCKET listener;
-    listener = open_socket (AF_INET, SOCK_STREAM, 0);
+    listener = zmq::open_socket (AF_INET, SOCK_STREAM, 0);
     wsa_assert (listener != INVALID_SOCKET);
 
     //  Set SO_REUSEADDR and TCP_NODELAY on listening socket.
@@ -430,15 +412,15 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
     memset (&addr, 0, sizeof addr);
     addr.sin_family = AF_INET;
     addr.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
-    addr.sin_port = htons (signaler_port);
+    addr.sin_port = htons (zmq::signaler_port);
 
     //  Create the writer socket.
-    *w_ = open_socket (AF_INET, SOCK_STREAM, 0);
+    *w_ = zmq::open_socket (AF_INET, SOCK_STREAM, 0);
     wsa_assert (*w_ != INVALID_SOCKET);
 
     if (sync != NULL) {
         //  Enter the critical section.
-        DWORD dwrc = WaitForSingleObject (sync, INFINITE);
+        const DWORD dwrc = WaitForSingleObject (sync, INFINITE);
         zmq_assert (dwrc == WAIT_OBJECT_0 || dwrc == WAIT_ABANDONED);
     }
 
@@ -446,7 +428,7 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
     rc = bind (listener, reinterpret_cast<const struct sockaddr *> (&addr),
                sizeof addr);
 
-    if (rc != SOCKET_ERROR && signaler_port == 0) {
+    if (rc != SOCKET_ERROR && zmq::signaler_port == 0) {
         //  Retrieve ephemeral port number
         int addrlen = sizeof addr;
         rc = getsockname (listener, reinterpret_cast<struct sockaddr *> (&addr),
@@ -454,25 +436,28 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
     }
 
     //  Listen for incoming connections.
-    if (rc != SOCKET_ERROR)
+    if (rc != SOCKET_ERROR) {
         rc = listen (listener, 1);
+    }
 
     //  Connect writer to the listener.
-    if (rc != SOCKET_ERROR)
+    if (rc != SOCKET_ERROR) {
         rc = connect (*w_, reinterpret_cast<struct sockaddr *> (&addr),
                       sizeof addr);
-
-    //  Set TCP_NODELAY on writer socket.
-    tune_socket (*w_);
+    }
 
     //  Accept connection from writer.
-    if (rc != SOCKET_ERROR)
+    if (rc != SOCKET_ERROR) {
+        //  Set TCP_NODELAY on writer socket.
+        tune_socket (*w_);
+
         *r_ = accept (listener, NULL, NULL);
+    }
 
     //  Send/receive large chunk to work around TCP slow start
     //  This code is a workaround for #1608
     if (*r_ != INVALID_SOCKET) {
-        size_t dummy_size =
+        const size_t dummy_size =
           1024 * 1024; //  1M to overload default receive buffer
         unsigned char *dummy =
           static_cast<unsigned char *> (malloc (dummy_size));
@@ -512,7 +497,7 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
     if (sync != NULL) {
         //  Exit the critical section.
         BOOL brc;
-        if (signaler_port == event_signaler_port)
+        if (zmq::signaler_port == event_signaler_port)
             brc = SetEvent (sync);
         else
             brc = ReleaseMutex (sync);
@@ -524,7 +509,7 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
     }
 
     if (*r_ != INVALID_SOCKET) {
-        make_socket_noninheritable (*r_);
+        zmq::make_socket_noninheritable (*r_);
         return 0;
     }
     //  Cleanup writer if connection failed
@@ -534,10 +519,150 @@ int zmq::make_fdpair (fd_t *r_, fd_t *w_)
         *w_ = INVALID_SOCKET;
     }
     //  Set errno from saved value
-    errno = wsa_error_to_errno (saved_errno);
+    errno = zmq::wsa_error_to_errno (saved_errno);
+    return -1;
+}
+#endif
+
+int zmq::make_fdpair (fd_t *r_, fd_t *w_)
+{
+#if defined ZMQ_HAVE_EVENTFD
+    int flags = 0;
+#if defined ZMQ_HAVE_EVENTFD_CLOEXEC
+    //  Setting this option result in sane behaviour when exec() functions
+    //  are used. Old sockets are closed and don't block TCP ports, avoid
+    //  leaks, etc.
+    flags |= EFD_CLOEXEC;
+#endif
+    fd_t fd = eventfd (0, flags);
+    if (fd == -1) {
+        errno_assert (errno == ENFILE || errno == EMFILE);
+        *w_ = *r_ = -1;
+        return -1;
+    }
+    *w_ = *r_ = fd;
+    return 0;
+
+
+#elif defined ZMQ_HAVE_WINDOWS
+#ifdef ZMQ_HAVE_IPC
+    ipc_address_t address;
+    std::string dirname, filename;
+    sockaddr_un lcladdr;
+    socklen_t lcladdr_len = sizeof lcladdr;
+    int rc = 0;
+    int saved_errno = 0;
+
+    // It appears that a lack of runtime AF_UNIX support
+    // can fail in more than one way.
+    // At least: open_socket can fail or later in bind
+    bool ipc_fallback_on_tcpip = true;
+
+    //  Create a listening socket.
+    const SOCKET listener = open_socket (AF_UNIX, SOCK_STREAM, 0);
+    if (listener == retired_fd) {
+        //  This may happen if the library was built on a system supporting AF_UNIX, but the system running doesn't support it.
+        goto try_tcpip;
+    }
+
+    create_ipc_wildcard_address (dirname, filename);
+
+    //  Initialise the address structure.
+    rc = address.resolve (filename.c_str ());
+    if (rc != 0) {
+        goto error_closelistener;
+    }
+
+    //  Bind the socket to the file path.
+    rc = bind (listener, const_cast<sockaddr *> (address.addr ()),
+               address.addrlen ());
+    if (rc != 0) {
+        errno = wsa_error_to_errno (WSAGetLastError ());
+        goto error_closelistener;
+    }
+    // if we got here, ipc should be working,
+    // so raise any remaining errors
+    ipc_fallback_on_tcpip = false;
+
+    //  Listen for incoming connections.
+    rc = listen (listener, 1);
+    if (rc != 0) {
+        errno = wsa_error_to_errno (WSAGetLastError ());
+        goto error_closelistener;
+    }
+
+    rc = getsockname (listener, reinterpret_cast<struct sockaddr *> (&lcladdr),
+                      &lcladdr_len);
+    wsa_assert (rc != -1);
+
+    //  Create the client socket.
+    *w_ = open_socket (AF_UNIX, SOCK_STREAM, 0);
+    if (*w_ == -1) {
+        errno = wsa_error_to_errno (WSAGetLastError ());
+        goto error_closelistener;
+    }
+
+    //  Connect to the remote peer.
+    rc = ::connect (*w_, reinterpret_cast<const struct sockaddr *> (&lcladdr),
+                    lcladdr_len);
+    if (rc == -1) {
+        goto error_closeclient;
+    }
+
+    *r_ = accept (listener, NULL, NULL);
+    errno_assert (*r_ != -1);
+
+    //  Close the listener socket, we don't need it anymore.
+    rc = closesocket (listener);
+    wsa_assert (rc == 0);
+
+    //  Cleanup temporary socket file descriptor
+    if (!filename.empty ()) {
+        rc = ::unlink (filename.c_str ());
+        if ((rc == 0) && !dirname.empty ()) {
+            rc = ::rmdir (dirname.c_str ());
+            dirname.clear ();
+        }
+        filename.clear ();
+    }
+
+    return 0;
+
+error_closeclient:
+    saved_errno = errno;
+    rc = closesocket (*w_);
+    wsa_assert (rc == 0);
+    errno = saved_errno;
+
+error_closelistener:
+    saved_errno = errno;
+    rc = closesocket (listener);
+    wsa_assert (rc == 0);
+
+    //  Cleanup temporary socket file descriptor
+    if (!filename.empty ()) {
+        rc = ::unlink (filename.c_str ());
+        if ((rc == 0) && !dirname.empty ()) {
+            rc = ::rmdir (dirname.c_str ());
+            dirname.clear ();
+        }
+        filename.clear ();
+    }
+
+    // ipc failed due to lack of AF_UNIX support, fallback on tcpip
+    if (ipc_fallback_on_tcpip) {
+        goto try_tcpip;
+    }
+
+    errno = saved_errno;
     return -1;
 
+try_tcpip:
+    // try to fallback to TCP/IP
+    // TODO: maybe remember this decision permanently?
+#endif
 
+    return make_fdpair_tcpip (r_, w_);
 #elif defined ZMQ_HAVE_OPENVMS
 
     //  Whilst OpenVMS supports socketpair - it maps to AF_INET only.  Further,
@@ -678,3 +803,155 @@ void zmq::make_socket_noninheritable (fd_t sock_)
     LIBZMQ_UNUSED (sock_);
 #endif
 }
+
+void zmq::assert_success_or_recoverable (zmq::fd_t s_, int rc_)
+{
+#ifdef ZMQ_HAVE_WINDOWS
+    if (rc_ != SOCKET_ERROR) {
+        return;
+    }
+#else
+    if (rc_ != -1) {
+        return;
+    }
+#endif
+
+    //  Check whether an error occurred
+    int err = 0;
+#if defined ZMQ_HAVE_HPUX || defined ZMQ_HAVE_VXWORKS
+    int len = sizeof err;
+#else
+    socklen_t len = sizeof err;
+#endif
+
+    const int rc = getsockopt (s_, SOL_SOCKET, SO_ERROR,
+                               reinterpret_cast<char *> (&err), &len);
+
+    //  Assert if the error was caused by 0MQ bug.
+    //  Networking problems are OK. No need to assert.
+#ifdef ZMQ_HAVE_WINDOWS
+    zmq_assert (rc == 0);
+    if (err != 0) {
+        wsa_assert (err == WSAECONNREFUSED || err == WSAECONNRESET
+                    || err == WSAECONNABORTED || err == WSAEINTR
+                    || err == WSAETIMEDOUT || err == WSAEHOSTUNREACH
+                    || err == WSAENETUNREACH || err == WSAENETDOWN
+                    || err == WSAENETRESET || err == WSAEACCES
+                    || err == WSAEINVAL || err == WSAEADDRINUSE);
+    }
+#else
+    //  Following code should handle both Berkeley-derived socket
+    //  implementations and Solaris.
+    if (rc == -1)
+        err = errno;
+    if (err != 0) {
+        errno = err;
+        errno_assert (errno == ECONNREFUSED || errno == ECONNRESET
+                      || errno == ECONNABORTED || errno == EINTR
+                      || errno == ETIMEDOUT || errno == EHOSTUNREACH
+                      || errno == ENETUNREACH || errno == ENETDOWN
+                      || errno == ENETRESET || errno == EINVAL);
+    }
+#endif
+}
+
+#ifdef ZMQ_HAVE_IPC
+
+#if defined ZMQ_HAVE_WINDOWS
+char *widechar_to_utf8 (const wchar_t *widestring)
+{
+    int nch, n;
+    char *utf8 = 0;
+    nch = WideCharToMultiByte (CP_UTF8, 0, widestring, -1, 0, 0, NULL, NULL);
+    if (nch > 0) {
+        utf8 = (char *) malloc ((nch + 1) * sizeof (char));
+        n = WideCharToMultiByte (CP_UTF8, 0, widestring, -1, utf8, nch, NULL,
+                                 NULL);
+        utf8[nch] = 0;
+    }
+    return utf8;
+}
+#endif
+
+int zmq::create_ipc_wildcard_address (std::string &path_, std::string &file_)
+{
+#if defined ZMQ_HAVE_WINDOWS
+    wchar_t buffer[MAX_PATH];
+
+    {
+        const errno_t rc = _wtmpnam_s (buffer);
+        errno_assert (rc == 0);
+    }
+
+    // TODO or use CreateDirectoryA and specify permissions?
+    const int rc = _wmkdir (buffer);
+    if (rc != 0) {
+        return -1;
+    }
+
+    char *tmp = widechar_to_utf8 (buffer);
+    if (tmp == 0) {
+        return -1;
+    }
+
+    path_.assign (tmp);
+    file_ = path_ + "/socket";
+
+    free (tmp);
+#else
+    std::string tmp_path;
+
+    // If TMPDIR, TEMPDIR, or TMP are available and are directories, create
+    // the socket directory there.
+    const char **tmp_env = tmp_env_vars;
+    while (tmp_path.empty () && *tmp_env != 0) {
+        const char *const tmpdir = getenv (*tmp_env);
+        struct stat statbuf;
+
+        // Confirm it is actually a directory before trying to use
+        if (tmpdir != 0 && ::stat (tmpdir, &statbuf) == 0
+            && S_ISDIR (statbuf.st_mode)) {
+            tmp_path.assign (tmpdir);
+            if (*(tmp_path.rbegin ()) != '/') {
+                tmp_path.push_back ('/');
+            }
+        }
+
+        // Try the next environment variable
+        ++tmp_env;
+    }
+
+    // Append a directory name
+    tmp_path.append ("tmpXXXXXX");
+
+    // We need room for tmp_path + trailing NUL
+    std::vector<char> buffer (tmp_path.length () + 1);
+    memcpy (&buffer[0], tmp_path.c_str (), tmp_path.length () + 1);
+
+#if defined HAVE_MKDTEMP
+    // Create the directory.  POSIX requires that mkdtemp() creates the
+    // directory with 0700 permissions, meaning the only possible race
+    // with socket creation could be the same user.  However, since
+    // each socket is created in a directory created by mkdtemp(), and
+    // mkdtemp() guarantees a unique directory name, there will be no
+    // collision.
+    if (mkdtemp (&buffer[0]) == 0) {
+        return -1;
+    }
+
+    path_.assign (&buffer[0]);
+    file_ = path_ + "/socket";
+#else
+    LIBZMQ_UNUSED (path_);
+    int fd = mkstemp (&buffer[0]);
+    if (fd == -1)
+        return -1;
+    ::close (fd);
+
+    file_.assign (&buffer[0]);
+#endif
+#endif
+
+    return 0;
+}
+#endif
