@@ -1,31 +1,4 @@
-/*
-    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
-
-    This file is part of libzmq, the ZeroMQ core engine in C++.
-
-    libzmq is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License (LGPL) as published
-    by the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    As a special exception, the Contributors give you permission to link
-    this library with independent modules to produce an executable,
-    regardless of the license terms of these independent modules, and to
-    copy and distribute the resulting executable under terms of your choice,
-    provided that you also meet, for each linked independent module, the
-    terms and conditions of the license of that module. An independent
-    module is a module which is not derived from or based on this library.
-    If you modify this library, you must extend this exception to your
-    version of the library.
-
-    libzmq is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
-    License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+/* SPDX-License-Identifier: MPL-2.0 */
 
 #include "testutil.hpp"
 #include "testutil_unity.hpp"
@@ -88,7 +61,46 @@ void test_ctx_shutdown ()
     // Close the socket.
     TEST_ASSERT_SUCCESS_ERRNO (zmq_close (socket));
 
-    // Destory the context, will now not hang as we have closed the socket.
+    // Destroy the context, will now not hang as we have closed the socket.
+    TEST_ASSERT_SUCCESS_ERRNO (zmq_ctx_destroy (ctx));
+}
+
+void test_ctx_shutdown_socket_opened_after ()
+{
+    //  Set up our context.
+    void *ctx = zmq_ctx_new ();
+    TEST_ASSERT_NOT_NULL (ctx);
+
+    // Open a socket to start context, and close it immediately again.
+    void *socket = zmq_socket (ctx, ZMQ_PULL);
+    TEST_ASSERT_NOT_NULL (socket);
+    TEST_ASSERT_SUCCESS_ERRNO (zmq_close (socket));
+
+    // Shutdown context.
+    TEST_ASSERT_SUCCESS_ERRNO (zmq_ctx_shutdown (ctx));
+
+    // Opening socket should now fail.
+    TEST_ASSERT_NULL (zmq_socket (ctx, ZMQ_PULL));
+    TEST_ASSERT_FAILURE_ERRNO (ETERM, -1);
+
+    // Destroy the context.
+    TEST_ASSERT_SUCCESS_ERRNO (zmq_ctx_destroy (ctx));
+}
+
+void test_ctx_shutdown_only_socket_opened_after ()
+{
+    //  Set up our context.
+    void *ctx = zmq_ctx_new ();
+    TEST_ASSERT_NOT_NULL (ctx);
+
+    // Shutdown context.
+    TEST_ASSERT_SUCCESS_ERRNO (zmq_ctx_shutdown (ctx));
+
+    // Opening socket should now fail.
+    TEST_ASSERT_NULL (zmq_socket (ctx, ZMQ_PULL));
+    TEST_ASSERT_FAILURE_ERRNO (ETERM, -1);
+
+    // Destroy the context.
     TEST_ASSERT_SUCCESS_ERRNO (zmq_ctx_destroy (ctx));
 }
 
@@ -123,8 +135,8 @@ struct poller_test_data_t
 
 void run_poller (void *data_)
 {
-    struct poller_test_data_t *poller_test_data =
-      (struct poller_test_data_t *) data_;
+    const poller_test_data_t *const poller_test_data =
+      static_cast<const poller_test_data_t *> (data_);
 
     void *socket =
       zmq_socket (poller_test_data->ctx, poller_test_data->socket_type);
@@ -201,6 +213,8 @@ int main (void)
     UNITY_BEGIN ();
     RUN_TEST (test_ctx_destroy);
     RUN_TEST (test_ctx_shutdown);
+    RUN_TEST (test_ctx_shutdown_socket_opened_after);
+    RUN_TEST (test_ctx_shutdown_only_socket_opened_after);
     RUN_TEST (test_zmq_ctx_term_null_fails);
     RUN_TEST (test_zmq_term_null_fails);
     RUN_TEST (test_zmq_ctx_shutdown_null_fails);
