@@ -1,31 +1,4 @@
-/*
-    Copyright (c) 2007-2017 Contributors as noted in the AUTHORS file
-
-    This file is part of libzmq, the ZeroMQ core engine in C++.
-
-    libzmq is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License (LGPL) as published
-    by the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    As a special exception, the Contributors give you permission to link
-    this library with independent modules to produce an executable,
-    regardless of the license terms of these independent modules, and to
-    copy and distribute the resulting executable under terms of your choice,
-    provided that you also meet, for each linked independent module, the
-    terms and conditions of the license of that module. An independent
-    module is a module which is not derived from or based on this library.
-    If you modify this library, you must extend this exception to your
-    version of the library.
-
-    libzmq is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
-    License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+/* SPDX-License-Identifier: MPL-2.0 */
 
 #include "testutil.hpp"
 #include "testutil_unity.hpp"
@@ -324,6 +297,7 @@ void call_poller_wait_all_null_event_fails_event_count_nonzero (void *poller_)
 
 void call_poller_wait_all_null_event_fails_event_count_zero (void *poller_)
 {
+    LIBZMQ_UNUSED (poller_);
 #if 0
     //  TODO this causes an assertion, which is not consistent if the number
     //  of events may be 0, the pointer should be allowed to by NULL in that
@@ -333,7 +307,10 @@ void call_poller_wait_all_null_event_fails_event_count_zero (void *poller_)
 }
 
 #define TEST_CASE_FUNC_PARAM(name, func)                                       \
-    void test_##name () { func (name); }
+    void test_##name ()                                                        \
+    {                                                                          \
+        func (name);                                                           \
+    }
 
 TEST_CASE_FUNC_PARAM (call_poller_wait_null_event_fails, test_with_valid_poller)
 TEST_CASE_FUNC_PARAM (call_poller_wait_all_null_event_fails_event_count_nonzero,
