@@ -1,31 +1,4 @@
-/*
-    Copyright (c) 2019 Contributors as noted in the AUTHORS file
-
-    This file is part of libzmq, the ZeroMQ core engine in C++.
-
-    libzmq is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License (LGPL) as published
-    by the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    As a special exception, the Contributors give you permission to link
-    this library with independent modules to produce an executable,
-    regardless of the license terms of these independent modules, and to
-    copy and distribute the resulting executable under terms of your choice,
-    provided that you also meet, for each linked independent module, the
-    terms and conditions of the license of that module. An independent
-    module is a module which is not derived from or based on this library.
-    If you modify this library, you must extend this exception to your
-    version of the library.
-
-    libzmq is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
-    License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+/* SPDX-License-Identifier: MPL-2.0 */
 
 #ifdef _WIN32
 #include "../src/windows.hpp"
@@ -104,6 +77,7 @@ int remote_connect (int socket, uint32_t addr, uint16_t port)
 void *setup_socks_server (char *socks_server_address,
                           int socks_server_address_len)
 {
+    LIBZMQ_UNUSED (socks_server_address_len);
     fprintf (stderr, "socks_server: setup socks server\n");
     int server_fd =
       bind_socket_resolve_port ("127.0.0.1", "0", socks_server_address);
@@ -871,7 +845,7 @@ void test_socks_userpass_options (void)
 
 int main ()
 {
-    setup_test_environment ();
+    setup_test_environment (180);
 
     UNITY_BEGIN ();
     RUN_TEST (test_socks_proxy_options);
