@@ -1,31 +1,4 @@
-/*
-    Copyright (c) 2007-2017 Contributors as noted in the AUTHORS file
-
-    This file is part of libzmq, the ZeroMQ core engine in C++.
-
-    libzmq is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License (LGPL) as published
-    by the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    As a special exception, the Contributors give you permission to link
-    this library with independent modules to produce an executable,
-    regardless of the license terms of these independent modules, and to
-    copy and distribute the resulting executable under terms of your choice,
-    provided that you also meet, for each linked independent module, the
-    terms and conditions of the license of that module. An independent
-    module is a module which is not derived from or based on this library.
-    If you modify this library, you must extend this exception to your
-    version of the library.
-
-    libzmq is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
-    License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+/* SPDX-License-Identifier: MPL-2.0 */
 
 #include "testutil.hpp"
 #include "testutil_unity.hpp"
@@ -168,6 +141,7 @@ void test_destroy_queue_on_disconnect (const char *bind_address_)
 // SHALL block on sending, or return a suitable error, when it has no connected peers.
 void test_block_on_send_no_peers (const char *bind_address_)
 {
+    LIBZMQ_UNUSED (bind_address_);
     void *sc = test_context_socket (ZMQ_DEALER);
 
     int timeout = 250;
@@ -185,7 +159,10 @@ void test_block_on_send_no_peers (const char *bind_address_)
     {                                                                          \
         test_round_robin_out (bind_address);                                   \
     }                                                                          \
-    void test_fair_queue_in_##name () { test_fair_queue_in (bind_address); }   \
+    void test_fair_queue_in_##name ()                                          \
+    {                                                                          \
+        test_fair_queue_in (bind_address);                                     \
+    }                                                                          \
     void test_block_on_send_no_peers_##name ()                                 \
     {                                                                          \
         test_block_on_send_no_peers (bind_address);                            \
