@@ -1,33 +1,6 @@
 #pragma once
 
-/*
-Copyright (c) 2018 Contributors as noted in the AUTHORS file
-
-This file is part of libzmq, the ZeroMQ core engine in C++.
-
-libzmq is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License (LGPL) as published
-by the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-
-As a special exception, the Contributors give you permission to link
-this library with independent modules to produce an executable,
-regardless of the license terms of these independent modules, and to
-copy and distribute the resulting executable under terms of your choice,
-provided that you also meet, for each linked independent module, the
-terms and conditions of the license of that module. An independent
-module is a module which is not derived from or based on this library.
-If you modify this library, you must extend this exception to your
-version of the library.
-
-libzmq is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
-License for more details.
-
-You should have received a copy of the GNU Lesser General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+/* SPDX-License-Identifier: MPL-2.0 */
 
 #include "../include/zmq.h"
 
@@ -197,8 +170,14 @@ void recv_array_expect_success (void *socket_,
 // Use this is an test executable to perform a default setup and teardown of
 // the test context, which is appropriate for many libzmq test cases.
 #define SETUP_TEARDOWN_TESTCONTEXT                                             \
-    void setUp () { setup_test_context (); }                                   \
-    void tearDown () { teardown_test_context (); }
+    void setUp ()                                                              \
+    {                                                                          \
+        setup_test_context ();                                                 \
+    }                                                                          \
+    void tearDown ()                                                           \
+    {                                                                          \
+        teardown_test_context ();                                              \
+    }
 
 // The maximum number of sockets that can be managed by the test context.
 #define MAX_TEST_SOCKETS 128
